searchData={"items":[{"type":"behaviour","title":"Smokestack","doc":"Smokestack provides a way to define test factories for your \n[Ash Resources](https://ash-hq.org/docs/module/ash/latest/ash-resource) \nusing a convenient DSL:\n\n```\ndefmodule MyApp.Factory do\n  use Smokestack\n\n  factory Character do\n    attribute :name, &Faker.StarWars.character/0\n    attribute :affiliation, choose([\"Galactic Empire\", \"Rebel Alliance\"])\n  end\nend\n\ndefmodule MyApp.CharacterTest do\n  use MyApp.DataCase\n  use MyApp.Factory\n\n  test \"it can build a character\" do\n    assert character = insert!(Character)\n  end\nend\n```","ref":"Smokestack.html"},{"type":"behaviour","title":"Templates - Smokestack","doc":"Each attribute uses a template to generate a value when building a factory.  \nTemplates can be anything that implements the `Smokestack.Template` protocol.\nThis protocol is automatically implemented for functions with arities zero\nthrough two - meaning you can just drop in your own functions - or use one of\nthe built-in helpers from `Smokestack.Dsl.Template`.","ref":"Smokestack.html#module-templates"},{"type":"behaviour","title":"Variants - Smokestack","doc":"Sometimes you need to make slightly different factories to build a resource\nin a specific state for your test scenario.  \n\nHere's an example defining an alternate `:trek` variant for the character\nfactory defined above:\n\n```\nfactory Character, :trek do\n  attribute :name, choose([\"J.L. Pipes\", \"Severn\", \"Slickback\"])\n  attribute :affiliation, choose([\"Entrepreneur\", \"Voyager\"])\nend\n```","ref":"Smokestack.html#module-variants"},{"type":"behaviour","title":"Building resource records - Smokestack","doc":"You can use `insert/2` and `insert!/2` to build and insert records.  Records\nare inserted using `Ash.Seed.seed/2`, which means that they bypass the usual\nAsh lifecycle (actions, validations, etc).\n\n#","ref":"Smokestack.html#module-building-resource-records"},{"type":"behaviour","title":"Options - Smokestack","doc":"- `load`: an atom, list of atoms or keyword list of the same listing \n  relationships, calculations and aggregates that should be loaded\n  after the record is created.\n- `count`: rather than inserting just a single record, you can specify a\n  number of records to be inserted.  A list of records will be returned.\n- `build`: an atom, list of atoms or keyword list of the same describing\n  relationships which you would like built alongside the record.  If the\n  related resource has a variant which matches the current one, it will be\n  used, and if not the `:default` variant will be.\n- `attrs`: A map or keyword list of attributes you would like to set directly\n  on the created record, rather than using the value provided by the factory.","ref":"Smokestack.html#module-options"},{"type":"behaviour","title":"Building parameters - Smokestack","doc":"As well as inserting records directly you can use `params/2` and `params!/2`\nto build parameters for use testing controller actions, HTTP requests, etc.\n\n#","ref":"Smokestack.html#module-building-parameters"},{"type":"behaviour","title":"Options - Smokestack","doc":"- `encode`: rather than returning a map or maps, provide an encoder module\n  to serialise the parameters.  Commonly you would use `Jason` or `Poison`.\n- `nest`: rather than returning a map or maps directly, wrap the result in\n  an outer map using the provided key.\n- `key_case`: change the case of the keys into one of the many cases\n  supported by [recase](https://hex.pm/packages/recase).\n- `key_type`: specify whether the returned map or maps should use string or\n  atom keys (ignored when using the `encode` option).\n- `count`: rather than returning just a single map, you can specify a\n  number of results to be returned.  A list of maps will be returned.\n- `build`: an atom, list of atoms or keyword list of the same describing\n  relationships which you would like built within the result.  If the\n  related resource has a variant which matches the current one, it will be\n  used, and if not the `:default` variant will be.\n- `attrs`: A map or keyword list of attributes you would like to set directly\n  on the result, rather than using the value provided by the factory.","ref":"Smokestack.html#module-options"},{"type":"behaviour","title":"DSL Documentation - Smokestack","doc":"#","ref":"Smokestack.html#module-dsl-documentation"},{"type":"behaviour","title":"Index - Smokestack","doc":"* [smokestack](#module-smokestack)\n   * factory\n     * attribute\n\n#","ref":"Smokestack.html#module-index"},{"type":"behaviour","title":"Docs - Smokestack","doc":"","ref":"Smokestack.html#module-docs"},{"type":"behaviour","title":"smokestack - Smokestack","doc":"* [factory](#module-factory)\n   * attribute\n\n\n\n\n\n---\n\n* `:api` (`t:atom/0`) - The default Ash API to use when evaluating loads\n\n\n\n#","ref":"Smokestack.html#module-smokestack"},{"type":"behaviour","title":"factory - Smokestack","doc":"Define factories for a resource\n\n * [attribute](#module-attribute)\n\n\n\n* `:api` (`t:atom/0`) - The Ash API to use when evaluating loads\n\n* `:resource` (`t:atom/0`) - Required. An Ash Resource\n\n* `:variant` (`t:atom/0`) - The name of a factory variant The default value is `:default`.\n\n\n\n###","ref":"Smokestack.html#module-factory"},{"type":"behaviour","title":"attribute - Smokestack","doc":"* `:name` (`t:atom/0`) - Required. The name of the target attribute\n\n* `:generator` - Required. A function which can generate an appropriate value for the attribute.œ","ref":"Smokestack.html#module-attribute"},{"type":"callback","title":"Smokestack.insert/2","doc":"Runs a factory and uses it to insert Ash resources into their data layers.\n\nAutomatically implemented by modules which `use Smokestack`.\n\nSee `Smokestack.RecordBuilder.build/3` for more information.","ref":"Smokestack.html#c:insert/2"},{"type":"callback","title":"Smokestack.insert!/2","doc":"Raising version of `insert/4`.\n\nAutomatically implemented by modules which `use Smokestack`.\n\nSee `Smokestack.RecordBuilder.build/3` for more information.","ref":"Smokestack.html#c:insert!/2"},{"type":"callback","title":"Smokestack.params/2","doc":"Runs a factory and uses it to build a parameters suitable for simulating a\nrequest.\n\nAutomatically implemented by modules which `use Smokestack`.\n\nSee `Smokestack.ParamBuilder.build/2` for more information.","ref":"Smokestack.html#c:params/2"},{"type":"callback","title":"Smokestack.params!/2","doc":"Raising version of `params/2`.\n\nAutomatically implemented by modules which `use Smokestack`.\n\nSee `Smokestack.ParamBuilder.build/3` for more information.","ref":"Smokestack.html#c:params!/2"},{"type":"type","title":"Smokestack.insert_option/0","doc":"","ref":"Smokestack.html#t:insert_option/0"},{"type":"type","title":"Smokestack.param_option/0","doc":"","ref":"Smokestack.html#t:param_option/0"},{"type":"type","title":"Smokestack.recursive_atom_list/0","doc":"","ref":"Smokestack.html#t:recursive_atom_list/0"},{"type":"type","title":"Smokestack.t/0","doc":"","ref":"Smokestack.html#t:t/0"},{"type":"type","title":"Smokestack.variant_option/0","doc":"Choose a factory variant to use.  Defaults to `:default`.","ref":"Smokestack.html#t:variant_option/0"},{"type":"behaviour","title":"Smokestack.Builder","doc":"A generic behaviour for \"building things\".","ref":"Smokestack.Builder.html"},{"type":"callback","title":"Smokestack.Builder.build/2","doc":"Given a Factory entity and some options build something.","ref":"Smokestack.Builder.html#c:build/2"},{"type":"function","title":"Smokestack.Builder.build/3","doc":"Given a builder and a factory, validate it's options and call the builder.","ref":"Smokestack.Builder.html#build/3"},{"type":"callback","title":"Smokestack.Builder.option_schema/1","doc":"Provide a schema for validating options.","ref":"Smokestack.Builder.html#c:option_schema/1"},{"type":"type","title":"Smokestack.Builder.error/0","doc":"","ref":"Smokestack.Builder.html#t:error/0"},{"type":"type","title":"Smokestack.Builder.result/0","doc":"","ref":"Smokestack.Builder.html#t:result/0"},{"type":"type","title":"Smokestack.Builder.t/0","doc":"","ref":"Smokestack.Builder.html#t:t/0"},{"type":"module","title":"Smokestack.Dsl","doc":"The DSL definition for the Smokestack DSL.","ref":"Smokestack.Dsl.html"},{"type":"module","title":"DSL Documentation - Smokestack.Dsl","doc":"#","ref":"Smokestack.Dsl.html#module-dsl-documentation"},{"type":"module","title":"Index - Smokestack.Dsl","doc":"* [smokestack](#module-smokestack)\n   * factory\n     * attribute\n\n#","ref":"Smokestack.Dsl.html#module-index"},{"type":"module","title":"Docs - Smokestack.Dsl","doc":"","ref":"Smokestack.Dsl.html#module-docs"},{"type":"module","title":"smokestack - Smokestack.Dsl","doc":"* [factory](#module-factory)\n   * attribute\n\n\n\n\n\n---\n\n* `:api` (`t:atom/0`) - The default Ash API to use when evaluating loads\n\n\n\n#","ref":"Smokestack.Dsl.html#module-smokestack"},{"type":"module","title":"factory - Smokestack.Dsl","doc":"Define factories for a resource\n\n * [attribute](#module-attribute)\n\n\n\n* `:api` (`t:atom/0`) - The Ash API to use when evaluating loads\n\n* `:resource` (`t:atom/0`) - Required. An Ash Resource\n\n* `:variant` (`t:atom/0`) - The name of a factory variant The default value is `:default`.\n\n\n\n###","ref":"Smokestack.Dsl.html#module-factory"},{"type":"module","title":"attribute - Smokestack.Dsl","doc":"* `:name` (`t:atom/0`) - Required. The name of the target attribute\n\n* `:generator` - Required. A function which can generate an appropriate value for the attribute.œ","ref":"Smokestack.Dsl.html#module-attribute"},{"type":"module","title":"Smokestack.Dsl.Attribute","doc":"The `attribute ` DSL entity.\n\nSee `d:Smokestack.factory.default.attribute` for more information.","ref":"Smokestack.Dsl.Attribute.html"},{"type":"type","title":"Smokestack.Dsl.Attribute.t/0","doc":"","ref":"Smokestack.Dsl.Attribute.html#t:t/0"},{"type":"module","title":"Smokestack.Dsl.Factory","doc":"The `factory` DSL entity.\n\nSee `d:Smokestack.factory` for more information.","ref":"Smokestack.Dsl.Factory.html"},{"type":"type","title":"Smokestack.Dsl.Factory.t/0","doc":"","ref":"Smokestack.Dsl.Factory.html#t:t/0"},{"type":"module","title":"Smokestack.Dsl.Info","doc":"Introspection of Smokestack DSLs.","ref":"Smokestack.Dsl.Info.html"},{"type":"function","title":"Smokestack.Dsl.Info.factory/3","doc":"Retrieve a variant for a specific resource.","ref":"Smokestack.Dsl.Info.html#factory/3"},{"type":"module","title":"Smokestack.Dsl.Template","doc":"Templates which assist in the generation of values.œ","ref":"Smokestack.Dsl.Template.html"},{"type":"function","title":"Smokestack.Dsl.Template.choose/2","doc":"Randomly select between a list of options.","ref":"Smokestack.Dsl.Template.html#choose/2"},{"type":"function","title":"Smokestack.Dsl.Template.constant/2","doc":"Select a constant value","ref":"Smokestack.Dsl.Template.html#constant/2"},{"type":"function","title":"Smokestack.Dsl.Template.cycle/2","doc":"Cycle sequentially between a list of options.","ref":"Smokestack.Dsl.Template.html#cycle/2"},{"type":"function","title":"Smokestack.Dsl.Template.n_times/3","doc":"Call a generator a number of times.","ref":"Smokestack.Dsl.Template.html#n_times/3"},{"type":"function","title":"Smokestack.Dsl.Template.sequence/2","doc":"Generate sequential values.","ref":"Smokestack.Dsl.Template.html#sequence/2"},{"type":"type","title":"Smokestack.Dsl.Template.element/0","doc":"","ref":"Smokestack.Dsl.Template.html#t:element/0"},{"type":"type","title":"Smokestack.Dsl.Template.mapper/0","doc":"","ref":"Smokestack.Dsl.Template.html#t:mapper/0"},{"type":"module","title":"Smokestack.FactoryBuilder","doc":"Executes a factory and returns it's result.","ref":"Smokestack.FactoryBuilder.html"},{"type":"function","title":"Smokestack.FactoryBuilder.build/2","doc":"Execute the named factory, if possible.","ref":"Smokestack.FactoryBuilder.html#build/2"},{"type":"type","title":"Smokestack.FactoryBuilder.attrs_option/0","doc":"Manually specify some attributes.","ref":"Smokestack.FactoryBuilder.html#t:attrs_option/0"},{"type":"type","title":"Smokestack.FactoryBuilder.error/0","doc":"","ref":"Smokestack.FactoryBuilder.html#t:error/0"},{"type":"type","title":"Smokestack.FactoryBuilder.option/0","doc":"","ref":"Smokestack.FactoryBuilder.html#t:option/0"},{"type":"type","title":"Smokestack.FactoryBuilder.result/0","doc":"","ref":"Smokestack.FactoryBuilder.html#t:result/0"},{"type":"module","title":"Smokestack.ManyBuilder","doc":"Handles repeatedly building.","ref":"Smokestack.ManyBuilder.html"},{"type":"function","title":"Smokestack.ManyBuilder.build/2","doc":"Run the factory a number of times.","ref":"Smokestack.ManyBuilder.html#build/2"},{"type":"type","title":"Smokestack.ManyBuilder.count_option/0","doc":"How many times should we call the builder?","ref":"Smokestack.ManyBuilder.html#t:count_option/0"},{"type":"type","title":"Smokestack.ManyBuilder.error/0","doc":"","ref":"Smokestack.ManyBuilder.html#t:error/0"},{"type":"type","title":"Smokestack.ManyBuilder.option/0","doc":"","ref":"Smokestack.ManyBuilder.html#t:option/0"},{"type":"type","title":"Smokestack.ManyBuilder.result/0","doc":"","ref":"Smokestack.ManyBuilder.html#t:result/0"},{"type":"module","title":"Smokestack.ParamBuilder","doc":"Handles the building of parameters.","ref":"Smokestack.ParamBuilder.html"},{"type":"function","title":"Smokestack.ParamBuilder.build/2","doc":"Run the factory and return a map or list-of-maps of params.","ref":"Smokestack.ParamBuilder.html#build/2"},{"type":"type","title":"Smokestack.ParamBuilder.encode_option/0","doc":"Encode the result using the specified encoder.\n\nSmokestack will call `encode/1` on the module provided with the generated\nresult (or results).  For example set `encode: Jason` or `encode: Poison`\nto encode the results as a JSON string.\n\nThe module's encode function should return an ok/error tuple.","ref":"Smokestack.ParamBuilder.html#t:encode_option/0"},{"type":"type","title":"Smokestack.ParamBuilder.error/0","doc":"","ref":"Smokestack.ParamBuilder.html#t:error/0"},{"type":"type","title":"Smokestack.ParamBuilder.key_case_option/0","doc":"Format the keys in the specified case. Defaults to `:snake`","ref":"Smokestack.ParamBuilder.html#t:key_case_option/0"},{"type":"type","title":"Smokestack.ParamBuilder.key_type_option/0","doc":"Convert the keys into the specified type.  Defaults to `:atom`.","ref":"Smokestack.ParamBuilder.html#t:key_type_option/0"},{"type":"type","title":"Smokestack.ParamBuilder.nest_option/0","doc":"Nest the result within the specified key in the output.","ref":"Smokestack.ParamBuilder.html#t:nest_option/0"},{"type":"type","title":"Smokestack.ParamBuilder.option/0","doc":"","ref":"Smokestack.ParamBuilder.html#t:option/0"},{"type":"type","title":"Smokestack.ParamBuilder.result/0","doc":"","ref":"Smokestack.ParamBuilder.html#t:result/0"},{"type":"module","title":"Smokestack.RecordBuilder","doc":"Handles the insertion of new records.","ref":"Smokestack.RecordBuilder.html"},{"type":"function","title":"Smokestack.RecordBuilder.build/2","doc":"Run the factory and insert a record, or records.","ref":"Smokestack.RecordBuilder.html#build/2"},{"type":"type","title":"Smokestack.RecordBuilder.error/0","doc":"","ref":"Smokestack.RecordBuilder.html#t:error/0"},{"type":"type","title":"Smokestack.RecordBuilder.load_option/0","doc":"A nested keyword list of associations, calculations and aggregates to load","ref":"Smokestack.RecordBuilder.html#t:load_option/0"},{"type":"type","title":"Smokestack.RecordBuilder.option/0","doc":"","ref":"Smokestack.RecordBuilder.html#t:option/0"},{"type":"type","title":"Smokestack.RecordBuilder.result/0","doc":"","ref":"Smokestack.RecordBuilder.html#t:result/0"},{"type":"module","title":"Smokestack.RelatedBuilder","doc":"Recursively build the factory and any related factories that have been\nrequested.","ref":"Smokestack.RelatedBuilder.html"},{"type":"function","title":"Smokestack.RelatedBuilder.build/2","doc":"Build related factories, if required.","ref":"Smokestack.RelatedBuilder.html#build/2"},{"type":"type","title":"Smokestack.RelatedBuilder.build_option/0","doc":"A nested keyword list of associations that should also be built.","ref":"Smokestack.RelatedBuilder.html#t:build_option/0"},{"type":"type","title":"Smokestack.RelatedBuilder.error/0","doc":"","ref":"Smokestack.RelatedBuilder.html#t:error/0"},{"type":"type","title":"Smokestack.RelatedBuilder.option/0","doc":"","ref":"Smokestack.RelatedBuilder.html#t:option/0"},{"type":"type","title":"Smokestack.RelatedBuilder.result/0","doc":"","ref":"Smokestack.RelatedBuilder.html#t:result/0"},{"type":"protocol","title":"Smokestack.Template","doc":"A protocol for generating values from templates.","ref":"Smokestack.Template.html"},{"type":"function","title":"Smokestack.Template.generate/3","doc":"Generate a value from the template.","ref":"Smokestack.Template.html#generate/3"},{"type":"function","title":"Smokestack.Template.init/1","doc":"Initialise the template, if required.","ref":"Smokestack.Template.html#init/1"},{"type":"type","title":"Smokestack.Template.mapper/0","doc":"","ref":"Smokestack.Template.html#t:mapper/0"},{"type":"type","title":"Smokestack.Template.t/0","doc":"All the types that implement this protocol.","ref":"Smokestack.Template.html#t:t/0"}],"content_type":"text/markdown"}